# Generated from D:/Projects/BESSER/besser/BUML/notations/action_language/BESSERActionLanguage.g4 by ANTLR 4.13.2
from antlr4 import *
if "." in __name__:
    from .BESSERActionLanguageParser import BESSERActionLanguageParser
else:
    from BESSERActionLanguageParser import BESSERActionLanguageParser

# This class defines a complete listener for a parse tree produced by BESSERActionLanguageParser.
class BESSERActionLanguageListener(ParseTreeListener):

    # Enter a parse tree produced by BESSERActionLanguageParser#function_definition.
    def enterFunction_definition(self, ctx:BESSERActionLanguageParser.Function_definitionContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#function_definition.
    def exitFunction_definition(self, ctx:BESSERActionLanguageParser.Function_definitionContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#parameter.
    def enterParameter(self, ctx:BESSERActionLanguageParser.ParameterContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#parameter.
    def exitParameter(self, ctx:BESSERActionLanguageParser.ParameterContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#parameterWithDefault.
    def enterParameterWithDefault(self, ctx:BESSERActionLanguageParser.ParameterWithDefaultContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#parameterWithDefault.
    def exitParameterWithDefault(self, ctx:BESSERActionLanguageParser.ParameterWithDefaultContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#statements.
    def enterStatements(self, ctx:BESSERActionLanguageParser.StatementsContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#statements.
    def exitStatements(self, ctx:BESSERActionLanguageParser.StatementsContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#cond_loop.
    def enterCond_loop(self, ctx:BESSERActionLanguageParser.Cond_loopContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#cond_loop.
    def exitCond_loop(self, ctx:BESSERActionLanguageParser.Cond_loopContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#while.
    def enterWhile(self, ctx:BESSERActionLanguageParser.WhileContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#while.
    def exitWhile(self, ctx:BESSERActionLanguageParser.WhileContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#do_while.
    def enterDo_while(self, ctx:BESSERActionLanguageParser.Do_whileContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#do_while.
    def exitDo_while(self, ctx:BESSERActionLanguageParser.Do_whileContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#for.
    def enterFor(self, ctx:BESSERActionLanguageParser.ForContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#for.
    def exitFor(self, ctx:BESSERActionLanguageParser.ForContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#iterator.
    def enterIterator(self, ctx:BESSERActionLanguageParser.IteratorContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#iterator.
    def exitIterator(self, ctx:BESSERActionLanguageParser.IteratorContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#conditional_branch.
    def enterConditional_branch(self, ctx:BESSERActionLanguageParser.Conditional_branchContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#conditional_branch.
    def exitConditional_branch(self, ctx:BESSERActionLanguageParser.Conditional_branchContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#block.
    def enterBlock(self, ctx:BESSERActionLanguageParser.BlockContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#block.
    def exitBlock(self, ctx:BESSERActionLanguageParser.BlockContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#condition.
    def enterCondition(self, ctx:BESSERActionLanguageParser.ConditionContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#condition.
    def exitCondition(self, ctx:BESSERActionLanguageParser.ConditionContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#assign_target.
    def enterAssign_target(self, ctx:BESSERActionLanguageParser.Assign_targetContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#assign_target.
    def exitAssign_target(self, ctx:BESSERActionLanguageParser.Assign_targetContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#explicit_declaration.
    def enterExplicit_declaration(self, ctx:BESSERActionLanguageParser.Explicit_declarationContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#explicit_declaration.
    def exitExplicit_declaration(self, ctx:BESSERActionLanguageParser.Explicit_declarationContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#assignment.
    def enterAssignment(self, ctx:BESSERActionLanguageParser.AssignmentContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#assignment.
    def exitAssignment(self, ctx:BESSERActionLanguageParser.AssignmentContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#return.
    def enterReturn(self, ctx:BESSERActionLanguageParser.ReturnContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#return.
    def exitReturn(self, ctx:BESSERActionLanguageParser.ReturnContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#type.
    def enterType(self, ctx:BESSERActionLanguageParser.TypeContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#type.
    def exitType(self, ctx:BESSERActionLanguageParser.TypeContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#single_type.
    def enterSingle_type(self, ctx:BESSERActionLanguageParser.Single_typeContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#single_type.
    def exitSingle_type(self, ctx:BESSERActionLanguageParser.Single_typeContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#sequence_type.
    def enterSequence_type(self, ctx:BESSERActionLanguageParser.Sequence_typeContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#sequence_type.
    def exitSequence_type(self, ctx:BESSERActionLanguageParser.Sequence_typeContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#optional_type.
    def enterOptional_type(self, ctx:BESSERActionLanguageParser.Optional_typeContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#optional_type.
    def exitOptional_type(self, ctx:BESSERActionLanguageParser.Optional_typeContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#function_type.
    def enterFunction_type(self, ctx:BESSERActionLanguageParser.Function_typeContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#function_type.
    def exitFunction_type(self, ctx:BESSERActionLanguageParser.Function_typeContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#any_type.
    def enterAny_type(self, ctx:BESSERActionLanguageParser.Any_typeContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#any_type.
    def exitAny_type(self, ctx:BESSERActionLanguageParser.Any_typeContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#real_type.
    def enterReal_type(self, ctx:BESSERActionLanguageParser.Real_typeContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#real_type.
    def exitReal_type(self, ctx:BESSERActionLanguageParser.Real_typeContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#string_type.
    def enterString_type(self, ctx:BESSERActionLanguageParser.String_typeContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#string_type.
    def exitString_type(self, ctx:BESSERActionLanguageParser.String_typeContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#int_type.
    def enterInt_type(self, ctx:BESSERActionLanguageParser.Int_typeContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#int_type.
    def exitInt_type(self, ctx:BESSERActionLanguageParser.Int_typeContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#bool_type.
    def enterBool_type(self, ctx:BESSERActionLanguageParser.Bool_typeContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#bool_type.
    def exitBool_type(self, ctx:BESSERActionLanguageParser.Bool_typeContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#nothing.
    def enterNothing(self, ctx:BESSERActionLanguageParser.NothingContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#nothing.
    def exitNothing(self, ctx:BESSERActionLanguageParser.NothingContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#classifier_type.
    def enterClassifier_type(self, ctx:BESSERActionLanguageParser.Classifier_typeContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#classifier_type.
    def exitClassifier_type(self, ctx:BESSERActionLanguageParser.Classifier_typeContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#expression.
    def enterExpression(self, ctx:BESSERActionLanguageParser.ExpressionContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#expression.
    def exitExpression(self, ctx:BESSERActionLanguageParser.ExpressionContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#ternary.
    def enterTernary(self, ctx:BESSERActionLanguageParser.TernaryContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#ternary.
    def exitTernary(self, ctx:BESSERActionLanguageParser.TernaryContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#boolean.
    def enterBoolean(self, ctx:BESSERActionLanguageParser.BooleanContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#boolean.
    def exitBoolean(self, ctx:BESSERActionLanguageParser.BooleanContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#or.
    def enterOr(self, ctx:BESSERActionLanguageParser.OrContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#or.
    def exitOr(self, ctx:BESSERActionLanguageParser.OrContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#and.
    def enterAnd(self, ctx:BESSERActionLanguageParser.AndContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#and.
    def exitAnd(self, ctx:BESSERActionLanguageParser.AndContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#equality.
    def enterEquality(self, ctx:BESSERActionLanguageParser.EqualityContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#equality.
    def exitEquality(self, ctx:BESSERActionLanguageParser.EqualityContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#comparison.
    def enterComparison(self, ctx:BESSERActionLanguageParser.ComparisonContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#comparison.
    def exitComparison(self, ctx:BESSERActionLanguageParser.ComparisonContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#instanceof.
    def enterInstanceof(self, ctx:BESSERActionLanguageParser.InstanceofContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#instanceof.
    def exitInstanceof(self, ctx:BESSERActionLanguageParser.InstanceofContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#arithmetic.
    def enterArithmetic(self, ctx:BESSERActionLanguageParser.ArithmeticContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#arithmetic.
    def exitArithmetic(self, ctx:BESSERActionLanguageParser.ArithmeticContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#plus_minus.
    def enterPlus_minus(self, ctx:BESSERActionLanguageParser.Plus_minusContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#plus_minus.
    def exitPlus_minus(self, ctx:BESSERActionLanguageParser.Plus_minusContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#mult_div.
    def enterMult_div(self, ctx:BESSERActionLanguageParser.Mult_divContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#mult_div.
    def exitMult_div(self, ctx:BESSERActionLanguageParser.Mult_divContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#remain.
    def enterRemain(self, ctx:BESSERActionLanguageParser.RemainContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#remain.
    def exitRemain(self, ctx:BESSERActionLanguageParser.RemainContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#Cast.
    def enterCast(self, ctx:BESSERActionLanguageParser.CastContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#Cast.
    def exitCast(self, ctx:BESSERActionLanguageParser.CastContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#Not.
    def enterNot(self, ctx:BESSERActionLanguageParser.NotContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#Not.
    def exitNot(self, ctx:BESSERActionLanguageParser.NotContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#NullCoalessing.
    def enterNullCoalessing(self, ctx:BESSERActionLanguageParser.NullCoalessingContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#NullCoalessing.
    def exitNullCoalessing(self, ctx:BESSERActionLanguageParser.NullCoalessingContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#Select.
    def enterSelect(self, ctx:BESSERActionLanguageParser.SelectContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#Select.
    def exitSelect(self, ctx:BESSERActionLanguageParser.SelectContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#Minus.
    def enterMinus(self, ctx:BESSERActionLanguageParser.MinusContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#Minus.
    def exitMinus(self, ctx:BESSERActionLanguageParser.MinusContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#ArrayAccess.
    def enterArrayAccess(self, ctx:BESSERActionLanguageParser.ArrayAccessContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#ArrayAccess.
    def exitArrayAccess(self, ctx:BESSERActionLanguageParser.ArrayAccessContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#FunctionCall.
    def enterFunctionCall(self, ctx:BESSERActionLanguageParser.FunctionCallContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#FunctionCall.
    def exitFunctionCall(self, ctx:BESSERActionLanguageParser.FunctionCallContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#FieldAccess.
    def enterFieldAccess(self, ctx:BESSERActionLanguageParser.FieldAccessContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#FieldAccess.
    def exitFieldAccess(self, ctx:BESSERActionLanguageParser.FieldAccessContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#Atom.
    def enterAtom(self, ctx:BESSERActionLanguageParser.AtomContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#Atom.
    def exitAtom(self, ctx:BESSERActionLanguageParser.AtomContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#field_access.
    def enterField_access(self, ctx:BESSERActionLanguageParser.Field_accessContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#field_access.
    def exitField_access(self, ctx:BESSERActionLanguageParser.Field_accessContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#array_access.
    def enterArray_access(self, ctx:BESSERActionLanguageParser.Array_accessContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#array_access.
    def exitArray_access(self, ctx:BESSERActionLanguageParser.Array_accessContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#function_call.
    def enterFunction_call(self, ctx:BESSERActionLanguageParser.Function_callContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#function_call.
    def exitFunction_call(self, ctx:BESSERActionLanguageParser.Function_callContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#atomic.
    def enterAtomic(self, ctx:BESSERActionLanguageParser.AtomicContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#atomic.
    def exitAtomic(self, ctx:BESSERActionLanguageParser.AtomicContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#procedure_call.
    def enterProcedure_call(self, ctx:BESSERActionLanguageParser.Procedure_callContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#procedure_call.
    def exitProcedure_call(self, ctx:BESSERActionLanguageParser.Procedure_callContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#this.
    def enterThis(self, ctx:BESSERActionLanguageParser.ThisContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#this.
    def exitThis(self, ctx:BESSERActionLanguageParser.ThisContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#new.
    def enterNew(self, ctx:BESSERActionLanguageParser.NewContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#new.
    def exitNew(self, ctx:BESSERActionLanguageParser.NewContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#literal.
    def enterLiteral(self, ctx:BESSERActionLanguageParser.LiteralContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#literal.
    def exitLiteral(self, ctx:BESSERActionLanguageParser.LiteralContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#single_literal.
    def enterSingle_literal(self, ctx:BESSERActionLanguageParser.Single_literalContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#single_literal.
    def exitSingle_literal(self, ctx:BESSERActionLanguageParser.Single_literalContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#int_literal.
    def enterInt_literal(self, ctx:BESSERActionLanguageParser.Int_literalContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#int_literal.
    def exitInt_literal(self, ctx:BESSERActionLanguageParser.Int_literalContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#string_literal.
    def enterString_literal(self, ctx:BESSERActionLanguageParser.String_literalContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#string_literal.
    def exitString_literal(self, ctx:BESSERActionLanguageParser.String_literalContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#bool_literal.
    def enterBool_literal(self, ctx:BESSERActionLanguageParser.Bool_literalContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#bool_literal.
    def exitBool_literal(self, ctx:BESSERActionLanguageParser.Bool_literalContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#real_literal.
    def enterReal_literal(self, ctx:BESSERActionLanguageParser.Real_literalContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#real_literal.
    def exitReal_literal(self, ctx:BESSERActionLanguageParser.Real_literalContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#null_literal.
    def enterNull_literal(self, ctx:BESSERActionLanguageParser.Null_literalContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#null_literal.
    def exitNull_literal(self, ctx:BESSERActionLanguageParser.Null_literalContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#enum_literal.
    def enterEnum_literal(self, ctx:BESSERActionLanguageParser.Enum_literalContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#enum_literal.
    def exitEnum_literal(self, ctx:BESSERActionLanguageParser.Enum_literalContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#sequence_literal.
    def enterSequence_literal(self, ctx:BESSERActionLanguageParser.Sequence_literalContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#sequence_literal.
    def exitSequence_literal(self, ctx:BESSERActionLanguageParser.Sequence_literalContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#range_literal.
    def enterRange_literal(self, ctx:BESSERActionLanguageParser.Range_literalContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#range_literal.
    def exitRange_literal(self, ctx:BESSERActionLanguageParser.Range_literalContext):
        pass


    # Enter a parse tree produced by BESSERActionLanguageParser#symbol.
    def enterSymbol(self, ctx:BESSERActionLanguageParser.SymbolContext):
        pass

    # Exit a parse tree produced by BESSERActionLanguageParser#symbol.
    def exitSymbol(self, ctx:BESSERActionLanguageParser.SymbolContext):
        pass



del BESSERActionLanguageParser